/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;


/**
 * <p>
 * Extends the <code>FileStatistics</code> class to provide the functionality to collect statistics of every
 * file/directory in a directory and output the generated report to console or a file. The processing of a
 * directory can be done either recursively or non-recursively.
 * </p>
 *
 * <p>
 * This class requires no additional configuration.
 * </p>
 *
 * <p>
 * The thread-safety of this class depends on the thread-safety of <code>FileStatistics</code> class.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 *
 * @see FileStatistics
 */
public class ExtendedFileStatistics extends FileStatistics {
    /**
     * <p>
     * Creates a new instance of <code>ExtendedFileStatistics</code> class.
     * </p>
     *
     * @see FileStatistics#FileStatistics()
     */
    public ExtendedFileStatistics() {
    }

    /**
     * <p>
     * Creates a new instance of <code>ExtendedFileStatistics</code> class. The configurations are loaded from the
     * given namespace.
     * </p>
     *
     * @param namespace the namespace where the configurations are loaded.
     *
     * @throws NullPointerException if <code>namespace</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>namespace</code> is empty string.
     * @throws ConfigurationException if error occurs when loading configurations; or mandatory property is missing;
     *         or invalid property value exists.
     *
     * @see FileStatistics#FileStatistics(String)
     */
    public ExtendedFileStatistics(String namespace) throws ConfigurationException {
        super(namespace);
    }

    /**
     * <p>
     * Processes every file in the given directory non-recursively. Any sub-directories in the given directory is
     * not processed. When error occurs during the process of the directory, the previously available statistical
     * results before the error are accumulated.
     * </p>
     *
     * @param directory the directory in which every file should be processed.
     *
     * @throws NullPointerException if <code>directory</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>directory</code> is not an existing directory.
     * @throws IOException if I/O error occurs.
     * @throws FileStatisticsException if any other error occurs when processing the directory.
     */
    public void processDirectory(File directory) throws IOException, FileStatisticsException {
        processDirectory(directory, false);
    }

    /**
     * <p>
     * Processes every file in the given directory. When the given flag is <code>true</code>, every sub-directory in
     * the given directory is also processed recursively. When error occurs during the process of the directory,
     * the previously available statistical results before the error are accumulated.
     * </p>
     *
     * @param directory the directory in which every file/directory should be processed.
     * @param recursive a flag indicating whether the sub-directories should be processed or not.
     *
     * @throws NullPointerException if <code>directory</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>directory</code> is not an existing directory.
     * @throws IOException if I/O error occurs.
     * @throws FileStatisticsException if any other error occurs when processing the directory.
     */
    public void processDirectory(File directory, boolean recursive)
        throws IOException, FileStatisticsException {
        Validator.validateNotNull(directory, "directory");

        if (!directory.isDirectory()) {
            // If the given file is not a directory.
            throw new IllegalArgumentException("The given directory is not an existing directory.");
        }

        File[] files = directory.listFiles();

        // Process every file/directory in the directory.
        for (int fileIndex = 0; fileIndex < files.length; ++fileIndex) {
            if (files[fileIndex].isDirectory()) {
                if (recursive) {
                    // If the file is a directory and need to process the directory recursively, do recursion.
                    processDirectory(files[fileIndex], true);
                }
            } else {
                // Otherwise, it must be a file.
                processFile(files[fileIndex]);
            }
        }
    }

    /**
     * <p>
     * Prints the generated statistics report to the console.
     * </p>
     *
     * @throws StatsReportingException if error occurs when rendering the statistics report.
     */
    public void printReport() throws StatsReportingException {
        System.out.print(getReporting().generateReport());
    }

    /**
     * <p>
     * Prints the generated statistics report to the given file.
     * </p>
     *
     * @param file the file which the generated statistics report is written to.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs when writing the file.
     * @throws StatsReportingException if error occurs when rendering the statistics report.
     */
    public void printReport(File file) throws IOException, StatsReportingException {
        PrintWriter writer = null;

        try {
            // Open the file.
            writer = new PrintWriter(new FileWriter(file));

            // Write the generated report.
            writer.print(getReporting().generateReport());
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }
}



