/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import java.io.File;


/**
 * <p>
 * Defines an interface with the functionality to aggregate the statistical results of processors and generated
 * statsitical report according to the aggregated result. The <code>aggregate</code> method is called each time when
 * a file processor collects statistical data from a file and generates the statistical result. Clearing the current
 * accumulated results is also provided by this interface. All files where statistical data are collected should be
 * located within the base directory if specified.
 * </p>
 *
 * <p>
 * In order to be used by <code>FileStatistics</code>, an implementation must provide a constructor with a
 * <code>Property</code> instance as argument where configuration properties are read from.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public interface StatsReporting {
    /**
     * <p>
     * Aggregates the given result processed by the given processor with the given file. The aggregated results
     * should be updated, however the given result is not modified.
     * </p>
     *
     * @param processor the file processor collecting the statistical data.
     * @param file the file where the statistical data are collected from.
     * @param result the statistical result generated by the file processor.
     *
     * @throws NullPointerException if <code>processor</code>, <code>file</code> or <code>result</code> is
     *         <code>null</code>.
     * @throws StatsReportingException if <code>file</code> is not located in the base directory when the base
     *         directory is specified.
     */
    public void aggregate(FileProcessor processor, File file, StatsResult result)
        throws StatsReportingException;

    /**
     * <p>
     * Generates a statistics report according to the current aggregated results.
     * </p>
     *
     * @return a statistics report of the current aggregated results.
     *
     * @throws StatsReportingException if any error occurs when generating the report.
     */
    public String generateReport() throws StatsReportingException;

    /**
     * <p>
     * Clears the current aggregated results. The base directory is specified if <code>baseDirectory</code> is not
     * <code>null</code>. A <code>null</code> value means the base directory is not specified. All processed files
     * should be located in the base directory if specified.
     * </p>
     *
     * @param baseDirectory the base directory where all processed files should be located in if not
     *        <code>null</code>.
     *
     * @throws IllegalArgumentException if <code>baseDirectory</code> is not a valid directory when it is not
     *         <code>null</code>.
     */
    public void reset(File baseDirectory);
}



