/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.StatsResult;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Defines a mapping of keys and statsitical results. The keys are non-empty strings. It provides common adding,
 * getting and clearing features. When adding an existing key to the mapping, the statisitical result is aggregated.
 * </p>
 *
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsCollection {
    /** Represents the mapping storing the keys and statistical results. */
    private Map stats = new HashMap();

    /**
     * <p>
     * Creates a new instance of <code>StatsCollection</code> class. Initially, the mapping is empty.
     * </p>
     */
    public StatsCollection() {
    }

    /**
     * <p>
     * Adds a statistical result to the collection using the given key. If the key already exists in the collection,
     * the given statistical result is aggregated to the previous result. Otherwise, a copy of the given statistical
     * result is stored using the given key.
     * </p>
     *
     * @param key the key of the statistical result to be stored.
     * @param stats the statistical result to be stored.
     *
     * @throws NullPointerException if <code>key</code> or <code>stats</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>key</code> is empty string.
     */
    public void add(String key, StatsResult stats) {
        Validator.validateNotNullOrEmpty(key, "key");
        Validator.validateNotNull(stats, "stats");

        StatsResult old = (StatsResult) this.stats.get(key);

        if (old == null) {
            // The key does not exist in the existing map, add a clone to the map.
            this.stats.put(key, new StatsResult(stats));
        } else {
            // Otherwise, aggregate the existing statistics
            old.aggregate(stats);
        }
    }

    /**
     * <p>
     * Gets the statistical result with the given key. A copy is returned. If the key does not exist in the
     * collection, <code>null</code> is returned.
     * </p>
     *
     * @param key the key of the statistical result to be retrieved.
     *
     * @return the statistical result with the given key if exists; <code>null</code> otherwise.
     *
     * @throws NullPointerException if <code>key</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>key</code> is empty string.
     */
    public StatsResult get(String key) {
        Validator.validateNotNullOrEmpty(key, "key");

        StatsResult result = (StatsResult) stats.get(key);

        // If not null, return a clone.
        return (result == null) ? null : new StatsResult(result);
    }

    /**
     * <p>
     * Gets all keys in this collection. A copy is returned. The list of keys are sorted in case-insensitive
     * alphabetical order.
     * </p>
     *
     * @return all keys in this collection.
     */
    public List getAllKeys() {
        List keys = new ArrayList(stats.keySet());

        // Sort the key using case-insensitive comparator.
        Collections.sort(keys, String.CASE_INSENSITIVE_ORDER);

        return keys;
    }

    /**
     * <p>
     * Clears all keys and statistical results stored in this collection.
     * </p>
     */
    public void clear() {
        stats.clear();
    }
}



