/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;

import com.topcoder.util.config.Property;

import java.io.File;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;

import java.util.Iterator;
import java.util.List;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;


/**
 * <p>
 * Defines a concrete implementation of <code>StatsReporting</code> interface to provide the functionality to render
 * statistical reports in XML. A XSLT style sheet may also be specified in order to transform the rendered XML
 * document into other proper format.
 * </p>
 *
 * <p>
 * The root element of the rendered XML document has the tag name 'report'. Under the root element, 'files',
 * 'directories', 'aliases' and 'overall' are the elements containing statistical results for files, directories and
 * file processors, and the overall result respectively. For each file, directory and file processor, 'file',
 * 'directory' and 'alias' are the elements containing the statistical result. The name of the files, directories
 * and aliases of file processors are rendered in the 'name' attribute of the corresponding element. For each metric
 * in the statistical result, the display name is used as the tag name and the text inside the element is the
 * quantity value of the metric. The display name of each metric must be a valid tag name of XML document. It is not
 * checked in this class.
 * </p>
 *
 * <p>
 * Property 'stylesheet' may exists in the configuration. When it exists, the value defines the file name of the XSLT
 * style sheet. When it is missing, the rendered XML document without transformation is the final output.
 * </p>
 *
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReporting extends AbstractStatsReporting {
    /** Represents the property name where the XSLT style sheet file name is loaded. */
    public static final String STYLESHEET_PROPERTY_KEY = "stylesheet";

    /** Represents the root element of the statistic report. */
    private static final String REPORT_ELEMENT = "report";

    /** Represents the element in the statistic report containing statistics of all file. */
    private static final String FILES_ELEMENT = "files";

    /** Represents the element in the statistic report containing statistics of one file. */
    private static final String FILE_ELEMENT = "file";

    /** Represents the element in the statistic report containing statistics of all directories. */
    private static final String DIRECTORIES_ELEMENT = "directories";

    /** Represents the element in the statistic report containing statistics of one directory. */
    private static final String DIRECTORY_ELEMENT = "directory";

    /** Represents the element in the statistic report containing statistics of all processors. */
    private static final String PROCESSORS_ELEMENT = "aliases";

    /** Represents the element in the statistic report containing statistics of one processor. */
    private static final String PROCESSOR_ELEMENT = "alias";

    /** Represents the element in the statistic report containing the overall statistics. */
    private static final String OVERALL_ELEMENT = "overall";

    /** Represents the attribute of names in the statistic report. */
    private static final String NAME_ATTRIBUTE = "name";

    /** Represents the XSLT style sheet file used to generate the statistical report. */
    private File stylesheet = null;

    /**
     * <p>
     * Creates a new instance of <code>XmlStatsReporting</code> class. The XSLT style sheet is not used and the
     * rendered XML document is the final result.
     * </p>
     */
    public XmlStatsReporting() {
    }

    /**
     * <p>
     * Creates a new instance of <code>XmlStatsReporting</code> class. The XSLT style sheet is specified.
     * </p>
     *
     * @param stylesheet the XSLT style sheet used to transform the rendered XML document.
     *
     * @throws NullPointerException if <code>stylesheet</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>stylesheet</code> is not a valid file.
     */
    public XmlStatsReporting(File stylesheet) {
        Validator.validateNotNull(stylesheet, "stylesheet");

        if (!stylesheet.isFile()) {
            throw new IllegalArgumentException("stylesheet must be an existing file.");
        }

        this.stylesheet = stylesheet;
    }

    /**
     * <p>
     * Creates a new instance of <code>XmlStatsReporting</code> class. The XSLT style sheet file is read from
     * configuration. When the property is missing, no transformation is performed.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if the XSLT style sheet file is specified, but it is not a valid file.
     */
    public XmlStatsReporting(Property property) throws ConfigurationException {
        String filename = property.getValue(STYLESHEET_PROPERTY_KEY);

        if (filename != null) {
            // If the file name is specified, it must be an existing file.
            stylesheet = new File(filename);

            if (!stylesheet.isFile()) {
                throw new ConfigurationException("Property " + STYLESHEET_PROPERTY_KEY + " must be an existing file.");
            }
        }
    }

    /**
     * <p>
     * Generates a statistics report according to the current aggregated results. The information includes the
     * overall statistical result and results for each file, directory and processor. The statistics report is
     * generated in XML format. If XSLT style sheet file exists, the XML document is transformed before return.
     * </p>
     *
     * @return a statistics report of the current aggregated results.
     *
     * @throws StatsReportingException if transforming the XML document using XSLT style sheet fails.
     */
    public String generateReport() throws StatsReportingException {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);

        // Render the XML header
        pw.println("<?xml version=\"1.0\"?>");

        // Render the open tag of root element.
        pw.println("<" + REPORT_ELEMENT + ">");

        // Create the files statistics. The file names are rendered in a case-insensitive order.
        renderGroup(getFileStats(), FILES_ELEMENT, FILE_ELEMENT, pw);

        // Create the directories statistics. The directory names are rendered in a case-insensitive order.
        renderGroup(getDirectoryStats(), DIRECTORIES_ELEMENT, DIRECTORY_ELEMENT, pw);

        // Create the processors statistics. The processor aliases are rendered in a case-insensitive order.
        renderGroup(getAliasStats(), PROCESSORS_ELEMENT, PROCESSOR_ELEMENT, pw);

        // Render the overall statistics
        pw.println("\t<" + OVERALL_ELEMENT + ">");
        renderStatsResult(getOverallStats(), "\t\t", pw);
        pw.println("\t</" + OVERALL_ELEMENT + ">");

        // Render the closing tag of root element.
        pw.println("</" + REPORT_ELEMENT + ">");

        pw.close();

        // Get the rendered XML string.
        String xml = sw.toString();

        if (stylesheet != null) {
            // If XSLT is available, transform the XML string.
            sw = new StringWriter();

            Source source = new StreamSource(new StringReader(xml));
            Result result = new StreamResult(sw);
            Source xsltSource = new StreamSource(stylesheet);

            try {
                Transformer transformer = TransformerFactory.newInstance().newTransformer(xsltSource);

                // Transform the source to the result using XSLT
                transformer.transform(source, result);
            } catch (TransformerConfigurationException e) {
                throw new StatsReportingException("XSLT transformer cannot be created.", e);
            } catch (TransformerException e) {
                throw new StatsReportingException("XSLT transformation fails.", e);
            }

            // Convert the result to XML
            xml = sw.toString();
        }

        return xml;
    }

    /**
     * <p>
     * Renders a statistics collection to the given print writer. The whole rendered collection is enclosed by one
     * XML element with the given group tag name. Each item in the collection is enclosed by one XML element with
     * the given item tag name. The XML string is formatted properly.
     * </p>
     *
     * @param stats the statistics collection to be rendered.
     * @param groupTag the group tag enclosing all items in the statistics collection.
     * @param itemTag the item tag enclosing one item in the statistics collection.
     * @param pw the print writer where the XML document is written to.
     */
    private void renderGroup(StatsCollection stats, String groupTag, String itemTag, PrintWriter pw) {
        // Write the open tag of group element.
        pw.println("\t<" + groupTag + ">");

        // Get the sorted keys.
        List keys = stats.getAllKeys();

        // Enumerate every sorted key in result statistics
        Iterator iter = keys.iterator();

        while (iter.hasNext()) {
            // Get the statistics for this item
            String key = (String) iter.next();
            StatsResult stat = stats.get(key);

            // Render the open tag of item element.
            pw.println("\t\t<" + itemTag + " " + NAME_ATTRIBUTE + "=\"" + key + "\">");

            // Render the result.
            renderStatsResult(stat, "\t\t\t", pw);

            // Render the close tag of item element.
            pw.println("\t\t</" + itemTag + ">");
        }

        // Render the close tag of group element.
        pw.println("\t</" + groupTag + ">");
    }

    /**
     * <p>
     * Renders metrics in a statistics result to the given print writer. For each metric, an indent is rendered
     * before the actual data.
     * </p>
     *
     * @param stat the statistics result to be rendered.
     * @param indent the indent of each metric in the XML string.
     * @param pw the print writer where the XML document is written to.
     */
    private void renderStatsResult(StatsResult stat, String indent, PrintWriter pw) {
        // Render every metric
        Iterator metricIter = stat.getMetrics().iterator();

        while (metricIter.hasNext()) {
            // Get the metric
            Metric metric = (Metric) metricIter.next();
            String displayName = metric.getDisplayName();

            // Render the metric and the statistic
            pw.println(indent + "<" + displayName + ">" + stat.getStat(metric) + "</" + displayName + ">");
        }
    }
}



