/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * Tests functionality and error cases of <code>StatsCollection</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class StatsCollectionTestCase extends TestCase {
    /** Represents the <code>StatsCollection</code> instance used in tests. */
    private StatsCollection stats;

    /** Represents the <code>StatsResult</code> instance used in tests. */
    private StatsResult result;

    /** Repesents the <code>Metric</code> instance used in tests. */
    private Metric metric;

    /**
     * <p>
     * Sets up the test environment. The test instances are created. The statsitical result contains one metric.
     * </p>
     */
    protected void setUp() {
        stats = new StatsCollection();
        result = new StatsResult();
        metric = new Metric("Name", "Display Name", "Description");

        result.setStat(metric, 100);
    }

    /**
     * <p>
     * Cleans up the test environment. The test instances are disposed.
     * </p>
     */
    protected void tearDown() {
        stats = null;
        result = null;
        metric = null;
    }

    /**
     * <p>
     * Tests <code>add</code> when the given key is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testAddKeyNull() {
        try {
            stats.add(null, result);
            fail("The key is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>add</code> when the given key is empty string. Expect IllegalArgumentException.
     * </p>
     */
    public void testAddKeyEmpty() {
        try {
            stats.add("  ", result);
            fail("The key is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>add</code> when the given statistical result is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testAddResultNull() {
        try {
            stats.add("key", null);
            fail("The statistical result is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>get</code> when the given key is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testGetNull() {
        try {
            stats.get(null);
            fail("The key is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>get</code> when the given key is empty string. Expect IllegalArgumentException.
     * </p>
     */
    public void testGetEmpty() {
        try {
            stats.get("  ");
            fail("The key is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>add</code> when the given key does not exists in the collection. The key should be
     * added to the collection associated with the given result. A copy of the given result should be made.
     * Therefore, modification to the original result should not affect the result in collection.
     * </p>
     */
    public void testAddNonExist() {
        stats.add("key", result);

        // Verify the result
        StatsResult inCollection = stats.get("key");

        assertEquals("The statistical result should be correctly added.", result.getStat(metric),
            inCollection.getStat(metric));

        // Verify a copy by setting the original result
        result.setStat(metric, 0);
        assertTrue("A copy of the statistical result should be made.",
            result.getStat(metric) != inCollection.getStat(metric));
    }

    /**
     * <p>
     * Tests accuracy of <code>add</code> when the given key exists in the collection. The given result should be
     * aggregated into the previous result.
     * </p>
     */
    public void testAddExist() {
        stats.add("key", result);
        result.setStat(metric, 20);

        // Add the existing key
        stats.add("key", result);

        // Verify the result
        StatsResult inCollection = stats.get("key");

        assertEquals("The statistical result should be correctly aggregated.", 120, inCollection.getStat(metric));
    }

    /**
     * <p>
     * Tests accuracy of <code>get</code> when the key does not exist in the collection. It should return
     * <code>null</code>. The accuracy test for normal cases is already done in the test cases of <code>add</code>
     * method.
     * </p>
     */
    public void testGetNonExist() {
        assertNull("When the key does not exist in the collection, null should be returned.", stats.get("key"));
    }

    /**
     * <p>
     * Tests accuracy of <code>getAllKeys</code>. The list of keys should be sorted case-insensitively.
     * </p>
     */
    public void testGetAllKeysAccuracy() {
        stats.add("bIn", result);
        stats.add("Two", result);
        stats.add("ZOO", result);
        stats.add("acrobat", result);

        // Get the list
        List list = stats.getAllKeys();
        List expected = new ArrayList();

        // The expected list should be sorted.
        expected.add("acrobat");
        expected.add("bIn");
        expected.add("Two");
        expected.add("ZOO");

        // Verify
        assertEquals("The key list should be sorted.", expected, list);
    }

    /**
     * <p>
     * Tests accuracy of <code>clear</code>. All keys and statistical results in the collection should be removed.
     * This can be verified by the empty list of keys.
     * </p>
     */
    public void testClearAccuracy() {
        stats.add("bIn", result);
        stats.add("Two", result);
        stats.add("ZOO", result);
        stats.add("acrobat", result);

        // Clear the collection
        stats.clear();

        // Get the list
        List list = stats.getAllKeys();

        // Verify the list should be empty
        assertTrue("The keys and statistical results should be removed.", list.isEmpty());
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(StatsCollectionTestCase.class);
    }
}



