/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.util.commandline.ArgumentValidationException;
import com.topcoder.util.commandline.CommandLineUtility;
import com.topcoder.util.commandline.IllegalSwitchException;
import com.topcoder.util.commandline.Switch;
import com.topcoder.util.commandline.UsageException;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.datavalidator.ObjectValidator;

import java.io.File;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Defines a command line interface for this component. <code>ExtendedFileStatistics</code> instances are used to
 * perform the task. When invalid argument is specified, a help message containing the usage of this command line
 * interface is printed to the console.
 * </p>
 *
 * <p>
 * The arguments of this command line interface has the form: '[-c config-file] [-n namespace] [-b base-path] [-r]
 * [-o output-file] path [path2 path3 ...]'. Switch '-c' specifies the configuration file used to configure the
 * component. By default, preloaded configurations are used. Switch '-n' specifies the configuration namespace used
 * to configure the component. By default, 'com.topcoder.file.statistics.Main' is used. When '-c' and '-n' both
 * exists, the configuration file must be XML format, and the configuration file is loaded to the specified
 * namespace. Switch '-b' specifies the base directory of all files/directories to be processed. When it is
 * specified, all paths should be relative to this directory. Otherwise, all paths are considered as absolute
 * paths. Switch '-r' specifies that the directories should be processed recursively. By default, the directories
 * are not processed recursively. Switch '-o' specifies the file where the generated statistics report is written
 * to. By default, the report is printed to the console. User must specify at least one file/directory to be
 * processed. Custom switches may be specified as defined in each file statistics, file processor, file matcher
 * and reporting implementation.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is stateless.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class Main {
    /** Represents the default namespace used by the command line interface if no namespace is specified. */
    public static final String NAMESPACE = "com.topcoder.file.statistics.Main";

    /** Represents the command line argument description. */
    private static final String COMMAND_LINE_ARGUMENT_DESCRIPTION = "java com.topcoder.file.statistics.Main "
        + "[-c config-file] [-n namespace] [-b base-path] [-r] [-o output-file] [-custom_switch value] "
        + "path [path path ...]";

    /** Represents the switch specifying the configuration file. */
    private static final String SWITCH_CONFIG_FILE = "c";

    /** Represents the switch specifying the namespace. */
    private static final String SWITCH_NAMESPACE = "n";

    /** Represents the switch specifying the base directory. */
    private static final String SWITCH_BASE_PATH = "b";

    /** Represents the switch specifying whether the directories are processed recursively. */
    private static final String SWITCH_RECURSIVE = "r";

    /** Represents the switch specifying the file name writing the statistic report. */
    private static final String SWITCH_OUTPUT_FILE = "o";

    /** Represents the description of the configuration file switch. */
    private static final String SWITCH_CONFIG_FILE_DESCRIPTION = "Specify the configuration file loaded before using "
        + "this component. By default, only preload configuration files are loaded.";

    /** Represents the description of the namespace switch. */
    private static final String SWITCH_NAMESPACE_DESCRIPTION = "Specify the namespace in configuration used to "
        + "configure this component. By default, 'com.topcoder.file.statistics.Main' is used.";

    /** Represents the description of the base directory switch. */
    private static final String SWITCH_BASE_PATH_DESCRIPTION = "Specify the base directory of all files/directories "
        + "being processed. The path(s) is/are expected to be relative to this directory. By default, all paths "
        + "should be absolute.";

    /** Represents the description of the resursive switch. */
    private static final String SWITCH_RECURSIVE_DESCRIPTION = "Specify the directories should be processed "
        + "recursively. By default, the directories are not recursively processed.";

    /** Represents the description of the output file switch. */
    private static final String SWITCH_OUTPUT_FILE_DESCRIPTION = "Specify the file where the statistics are written "
        + "to. By default, the statistics are written to the console.";

    /** Represents the error message when invalid switches are specified. */
    private static final String INVALID_SWITCH_MESSAGE = "Invalid switch specified in command line.";

    /** Represents the error message when no path is specified. */
    private static final String NO_PATH_MESSAGE = "No file/directory to be processed.";

    /**
     * Represents the error message when a mandatory switch is missing.
     *
     * @since 2.0b
     */
    private static final String SWITCH_MISSING_ERROR = " is required, but is missing in the command line.";

    /**
     * Represents the error message when a switch's value is invalid.
     *
     * @since 2.0b
     */
    private static final String SWITCH_INVALID_ERROR = "'s value is invalid in the command line.";

    /**
     * <p>
     * Creates a new instance of <code>Main</code> class. This private constructor prevents from creating a new
     * instance of this class.
     * </p>
     */
    private Main() {
    }

    /**
     * <p>
     * Defines the command line interface entry. It configures the <code>ExtendedFileStatistics</code> instance
     * according to the specified arguments, and processes the files/directories in the arguments.
     * </p>
     *
     * @param args the command line arguments containing configuration information and files/directories to be
     *        processed.
     */
    public static void main(String[] args) {
        CommandLineUtility utility = new CommandLineUtility(true, 0, -1);

        // Add predefined switches
        addPredefinedSwitches(utility);

        try {
            // Parse the command line
            // The initial parsing only involves pre-defined switches which may affect the instantiation
            // of FileStatistics instance.
            utility.parse(args);
        } catch (NullPointerException e) {
            // Sometimes, an invalid switch causes undocumented NullPointerException.
            System.out.println(INVALID_SWITCH_MESSAGE);
            printHelp(utility);

            return;
        } catch (UsageException e) {
            // Ignore here. More validation is done after the creation of FileStatistics instance.
        } catch (ArgumentValidationException e) {
            // No validation
        }

        // Get list of files being processed.
        List files = utility.getParameters();

        if (files.isEmpty()) {
            // Must specify at least one file/directory.
            System.out.println(NO_PATH_MESSAGE);
            printHelp(utility);

            return;
        }

        // Get valid switches and configure the instance.
        List validSwitches = utility.getValidSwitches();
        String namespace = null;
        String configFile = null;
        File outputFile = null;
        File baseDir = null;
        boolean recursive = false;

        // Enumerate every specified switch
        Iterator switchIter = validSwitches.iterator();

        while (switchIter.hasNext()) {
            Switch specified = (Switch) switchIter.next();
            String name = specified.getName();

            // Modify default values
            if (name.equals(SWITCH_BASE_PATH)) {
                baseDir = new File(specified.getValue());
            } else if (name.equals(SWITCH_CONFIG_FILE)) {
                configFile = specified.getValue();
            } else if (name.equals(SWITCH_NAMESPACE)) {
                namespace = specified.getValue();
            } else if (name.equals(SWITCH_OUTPUT_FILE)) {
                outputFile = new File(specified.getValue());
            } else {
                recursive = true;
            }
        }

        try {
            if (configFile != null) {
                // Need to load configuration file
                if (namespace != null) {
                    // If the configuration namespace is specified, load the configuration to the namespace.
                    ConfigManager.getInstance().add(namespace, configFile, ConfigManager.CONFIG_XML_FORMAT);
                } else {
                    // Otherwise, load the configuration directly.
                    ConfigManager.getInstance().add(configFile);
                }
            }

            if (namespace == null) {
                // If namespace is not specified, use the default namespace.
                namespace = NAMESPACE;
            }

            // Create an instance used to process files
            ExtendedFileStatistics fileStatistics = new ExtendedFileStatistics(namespace);

            // Process the arguments
            if (!processArguments(args, fileStatistics)) {
                // If error occurs when processing the argument, quit.
                return;
            }

            // Set base directory
            fileStatistics.getReporting().reset(baseDir);

            // Enumerate every specified file or directory
            Iterator iter = files.iterator();

            while (iter.hasNext()) {
                File file = new File(baseDir, (String) iter.next());

                if (file.isDirectory()) {
                    // If the file is a directory, process it recursively or not according to the flag.
                    fileStatistics.processDirectory(file, recursive);
                } else {
                    // Otherwise, try to process it as a file.
                    // If the file does not exist, IOException is thrown by processors.
                    fileStatistics.processFile(file);
                }
            }

            if (outputFile != null) {
                // Print to file, if specified output file.
                fileStatistics.printReport(outputFile);
            } else {
                // Otherwise, print to console.
                fileStatistics.printReport();
            }
        } catch (Exception e) {
            System.out.print("Error occurred: " + e.getMessage());

            Throwable cause = e.getCause();

            // Print all original exception error messages, since Java does not do it automatically
            while (cause != null) {
                System.out.print(" caused by " + cause.getMessage());
                cause = cause.getCause();
            }

            System.out.println();
        }
    }

    /**
     * <p>
     * Adds predefined switches to the given command line utility.
     * </p>
     *
     * @param utility the command line utility where the predefined switches are added.
     */
    private static void addPredefinedSwitches(CommandLineUtility utility) {
        try {
            // Add valid switches to the command line utility.
            utility.addSwitch(new Switch(SWITCH_BASE_PATH, false, 1, 1, null, SWITCH_BASE_PATH_DESCRIPTION));
            utility.addSwitch(new Switch(SWITCH_CONFIG_FILE, false, 1, 1, null, SWITCH_CONFIG_FILE_DESCRIPTION));
            utility.addSwitch(new Switch(SWITCH_NAMESPACE, false, 1, 1, null, SWITCH_NAMESPACE_DESCRIPTION));
            utility.addSwitch(new Switch(SWITCH_OUTPUT_FILE, false, 1, 1, null, SWITCH_OUTPUT_FILE_DESCRIPTION));
            utility.addSwitch(new Switch(SWITCH_RECURSIVE, false, 0, 0, null, SWITCH_RECURSIVE_DESCRIPTION));
        } catch (IllegalSwitchException e) {
            // Impossible.
        }
    }

    /**
     * <p>
     * Processes the arguments specified in the command line. The arguments are collected and validated before sending
     * them to the corresponding handler. If validation error occurs, the error message is printed as well as the usage
     * help.
     * </p>
     *
     * @param args the command line arguments
     * @param fileStatistics the <code>FileStatistics</code> instance to be configured using the command line.
     * @return <code>true</code> if no error occurs when processing the arguments; <code>false</code> otherwise.
     */
    private static boolean processArguments(String[] args, FileStatistics fileStatistics) {
        // Get the argument handlers
        CommandLineUtility utility = new CommandLineUtility();

        addPredefinedSwitches(utility);

        List handlers = fileStatistics.getArgumentHandlers();
        List options = new ArrayList();
        Iterator iter;

        // Enumerate every handler and get the list of options
        iter = handlers.iterator();

        while (iter.hasNext()) {
            ArgumentHandler handler = (ArgumentHandler) iter.next();

            options.addAll(handler.proposeOptions());
        }

        // Add all custom switches to the command line utility
        iter = options.iterator();

        while (iter.hasNext()) {
            ArgumentOption option = (ArgumentOption) iter.next();

            try {
                utility.addSwitch(new Switch(option.getSwitchName(), option.isRequired(), 0, 1, null));
            } catch (IllegalSwitchException e) {
                // ignore the illegal switches, since multiple argument options may have the same name
            }
        }

        // Parse the command line again with all switches, including custom switches.
        try {
            // Parse the command line
            utility.parse(args);
        } catch (NullPointerException e) {
            // Sometimes, an invalid switch causes undocumented NullPointerException.
            System.out.println(INVALID_SWITCH_MESSAGE);
            printHelp(utility);

            return false;
        } catch (UsageException e) {
            // Illegal usage for some switches.
            System.out.println(INVALID_SWITCH_MESSAGE);
            printHelp(utility);

            return false;
        } catch (ArgumentValidationException e) {
            // No validation
        }

        // The map of switch names and values, both registered and custom switches
        Map arguments = getArguments(utility.getValidSwitches());

        try {
            // Validate every option
            iter = options.iterator();

            while (iter.hasNext()) {
                ArgumentOption option = (ArgumentOption) iter.next();
                String name = option.getSwitchName();

                if (!arguments.containsKey(name)) {
                    if (option.isRequired()) {
                        // The option is required, but missing.
                        throw new MissingInitializationException(name + SWITCH_MISSING_ERROR);
                    }
                } else {
                    ObjectValidator validator = option.getValidator();

                    if (validator != null) {
                        // If a validator exists, the value should be validated.
                        if (!validator.valid(arguments.get(name))) {
                            throw new MissingInitializationException(name + SWITCH_INVALID_ERROR);
                        }
                    }
                }
            }
        } catch (MissingInitializationException e) {
            // Validation error occurs.
            System.out.println(e.getMessage());
            printHelp(utility);

            return false;
        }

        // Pass the value to the handler
        iter = handlers.iterator();

        while (iter.hasNext()) {
            ArgumentHandler handler = (ArgumentHandler) iter.next();
            List handlerOption = handler.proposeOptions();
            Map map = new HashMap();

            // Copy only the options proposed by the handler
            Iterator optionIter = handlerOption.iterator();

            while (optionIter.hasNext()) {
                ArgumentOption option = (ArgumentOption) optionIter.next();
                String name = option.getSwitchName();

                map.put(name, arguments.get(name));
            }

            try {
                // Process the argument by the handler
                handler.handleArguments(map);
            } catch (InvalidArgumentException e) {
                // Validation fails, print the error message and the usage help.
                System.out.println(e.getMessage());
                printHelp(utility);

                return false;
            }
        }

        return true;
    }

    /**
     * <p>
     * Gets a map of switch names and switch values from the given list of switches. If the switch value is not
     * specified, empty string (length 0) is used as the value.
     * </p>
     *
     * @param switches the list of switches from command line utility.
     * @return a map of switch names and switch values.
     */
    private static Map getArguments(List switches) {
        Iterator iter = switches.iterator();
        Map arguments = new HashMap();

        // Enumerate every switch
        while (iter.hasNext()) {
            Switch s = (Switch) iter.next();

            // If the value is null, change it to "".
            arguments.put(s.getName(), (s.getValue() == null) ? "" : s.getValue());
        }

        return arguments;
    }

    /**
     * <p>
     * Prints the help messages containing the usage of this component to the console when invalid argument is
     * specified.
     * </p>
     *
     * @param utility the command line utility instance containing definitions of switches.
     */
    private static void printHelp(CommandLineUtility utility) {
        System.out.println(COMMAND_LINE_ARGUMENT_DESCRIPTION);
        System.out.println(utility.getUsageString());
    }
}




