/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

/**
 * <p>
 * Defines a statistical measurement of a file, such as the length, the number of lines, etc. Each metric has a name,
 * a display name and a description. Two metrics has the same name are considered as the same metric. When
 * aggregating the statistics, the statistical data of the same metric are accumulated. In the generated report, the
 * display name of the metric is used instead of the name. The name, display name and description cannot be empty
 * string after trimming.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is immutable.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class Metric {
    /** Represents the name of the metric. */
    private String name;

    /** Represents the display name of the metric used in generated report. */
    private String displayName;

    /** Represents the description of the metric. Currently, it is not used in this component. */
    private String description;

    /**
     * <p>
     * Creates a new instance of <code>Metric</code> class. The name, display name and description of the metric are
     * given.
     * </p>
     *
     * @param name the name of the metric.
     * @param displayName the display name of the metric.
     * @param description the description of the metric.
     *
     * @throws NullPointerException if <code>name</code>, <code>displayName</code> or <code>description</code> is
     *         <code>null</code>.
     * @throws IllegalArgumentException if <code>name</code>, <code>displayName</code> or <code>description</code> is
     *         empty string.
     */
    public Metric(String name, String displayName, String description) {
        Validator.validateNotNullOrEmpty(name, "name");
        Validator.validateNotNullOrEmpty(displayName, "displayName");
        Validator.validateNotNullOrEmpty(description, "description");

        this.name = name;
        this.displayName = displayName;
        this.description = description;
    }

    /**
     * <p>
     * Gets the name of the metric.
     * </p>
     *
     * @return the name of the metric.
     */
    public String getName() {
        return name;
    }

    /**
     * <p>
     * Gets the display name of the metric.
     * </p>
     *
     * @return the display name of the metric.
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * <p>
     * Gets the description of the metric.
     * </p>
     *
     * @return the description of the metric.
     */
    public String getDescription() {
        return description;
    }

    /**
     * <p>
     * Gets a flag indicating whether an object equals to this metric instance. An object equals to this metric
     * instance if and only if when the given object is an instance of <code>Metric</code> class, and its name is
     * the same as the name of this metric.
     * </p>
     *
     * @param object the object instance to be compared with.
     *
     * @return <code>true</code> if the object is an instance of <code>Metric</code>, and its name is the same as the
     *         name of this metric; <code>false</code> otherwise.
     */
    public boolean equals(Object object) {
        return (object instanceof Metric) ? ((Metric) object).name.equals(name) : false;
    }

    /**
     * <p>
     * Gets the hash code of this metric. The hash code is the same as the hash code of the metric's name.
     * </p>
     *
     * @return the hash code of this metric.
     */
    public int hashCode() {
        return name.hashCode();
    }
}




