/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;


/**
 * <p>
 * Defines an abstract implementation of <code>FileMatcher</code> interface to provide the functionality to match a
 * file according to its content category (either text or binary). Concrete implementations should only override the
 * abstract method. A file is considered as text file if the first 1024 bytes of the file contain only printable
 * ASCII characters (i.e. ASCII code from 32 to 127), tab, carriage return and line feed. Otherwise, it is
 * considered as binary file.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is stateless.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public abstract class FileCategoryMatcher implements FileMatcher {
    /** Represents the max number of bytes to be checked. */
    private static final int MAX_BUFFER_SIZE = 1024;

    /** Represents the carriage return character. */
    private static final byte CR = 13;

    /** Represents the line feed character. */
    private static final byte LF = 10;

    /** Represents the tab character. */
    private static final byte TAB = 7;

    /** Represents the lower bound of the character considered as text. */
    private static final byte MIN_TEXT = 32;

    /**
     * <p>
     * Creates a new instance of <code>FileCategoryMatcher</code> class.
     * </p>
     */
    protected FileCategoryMatcher() {
    }

    /**
     * <p>
     * Gets a flag indicating whether the given file is binary. The file is considered binary if its first 1024 bytes
     * contain characters other than printable ASCII characters (i.e. ASCII code 32 to 127), tab, carriage return
     * and line feed.
     * </p>
     *
     * @param file the file to be checked.
     *
     * @return a flag indicating whether the given file is binary.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     */
    public static boolean isBinary(File file) throws IOException {
        Validator.validateNotNull(file, "file");

        InputStream input = null;
        byte[] content = new byte[MAX_BUFFER_SIZE];
        boolean binary = false;

        try {
            input = new FileInputStream(file);

            // Read at most 1024 bytes from file.
            int size = input.read(content);

            // Check each byte in the file.
            for (int i = 0; i < size; ++i) {
                if ((content[i] != CR) && (content[i] != LF) && (content[i] != TAB) && (content[i] < MIN_TEXT)) {
                    // Check for binary characters. Since the maximum value of byte is 127,
                    // no need to check the upper bound.
                    binary = true;

                    break;
                }
            }
        } finally {
            if (input != null) {
                input.close();
            }
        }

        return binary;
    }

    /**
     * <p>
     * Gets a flag indicating whether the given file matches the criteria defined by the specific implementation. The
     * criteria is based on whether the file is binary or text.
     * </p>
     *
     * @param file the file to be matched.
     *
     * @return <code>true</code> if the given file matches the criteria; <code>false</code> otherwise.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     */
    public boolean match(File file) throws IOException {
        return match(isBinary(file));
    }

    /**
     * <p>
     * Gets a flag indicating whether the binary file flag matches the implementation's criteria.
     * </p>
     *
     * @param isBinary a flag indicating whether the file is binary.
     *
     * @return <code>true</code> if the given flag matches the criteria; <code>false</code> otherwise.
     */
    protected abstract boolean match(boolean isBinary);
}

