/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileMatchingException;

import java.io.File;


/**
 * <p>
 * Defines an abstract implementation of <code>FileMatcher</code> interface to provide the functionality to match
 * files according to the file name. Concrete implementations should only override the abstract method.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is stateless.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public abstract class FileNameMatcher implements FileMatcher {
    /**
     * <p>
     * Creates a new instance of <code>FileNameMatcher</code> class.
     * </p>
     */
    protected FileNameMatcher() {
    }

    /**
     * <p>
     * Gets a flag indicating whether the given file matches the criteria defined by the specific implementation. The
     * criteria is based on the file name.
     * </p>
     *
     * @param file the file to be matched.
     *
     * @return <code>true</code> if the given file matches the criteria; <code>false</code> otherwise.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws FileMatchingException if any error occurs when matching the file.
     */
    public boolean match(File file) throws FileMatchingException {
        Validator.validateNotNull(file, "file");

        return match(file.getName());
    }

    /**
     * <p>
     * Gets a flag indicating whether the given file name matches the criteria defined by the specific
     * implementation.
     * </p>
     *
     * @param filename the file name to be matched.
     *
     * @return <code>true</code> if the file name matches the criteria; <code>false</code> otherwise.
     *
     * @throws FileMatchingException if any error occurs when matching the file name.
     */
    protected abstract boolean match(String filename) throws FileMatchingException;
}




