/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ArgumentHandler;
import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;

import com.topcoder.util.config.Property;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Defines a concrete implementation of <code>FileProcessor</code> interface with the functionality to count number
 * of lines for files. The statistical result is stored using the metric named 'Line Count'. Lines are separated by
 * carriage return (known as CR), line feed (known as LF), or carriage return immediately following line feed (known
 * as CRLF). It has an optional configuration which can ignore the blank lines when counting the line number (blank
 * lines are considered as lines with whitespaces only).
 * </p>
 *
 * <p>
 * Property 'ignore_blank_line' can exist under the given property. When the value is 'on', 'yes' or 'true'
 * (case-insensitive), the blank lines are ignored. Otherwise, the blank lines are counted. By default, the blank
 * lines are counted. When specifying from command line, if the switch 'ignore_blank_line' exists, the blank lines
 * are ignored. Otherwise, the blank lines are counted.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is stateless and its super class is also thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 *
 * @see FileProcessorBase
 */
public class TextLineCounter extends FileProcessorBase implements ArgumentHandler {
    /** Represents the name of the metric supported by this processor. */
    public static final String NAME = "Line Count";

    /** Represents the display name of the metric supported by this processor. */
    public static final String DISPLAY_NAME = "line";

    /**
     * Represents the property name where the flag indicating whether blank lines should be ignored or not is loaded.
     *
     * @since 2.0b
     */
    public static final String IGNORE_BLANK_LINE_PROPERTY_KEY = "ignore_blank_line";

    /** Represents the description of this processor. */
    private static final String DESCRIPTION = "Count the number of lines for a text file.";

    /** Represents the metric describing the feature of this processor. */
    private static final Metric SUPPORTED_METRIC = new Metric(NAME, DISPLAY_NAME, DESCRIPTION);

    /**
     * Represents the flag indicating whether blank lines should be ignored.
     *
     * @since 2.0b
     */
    private boolean ignoreBlankLine = false;

    /**
     * Represents the property value of matching all processors flag representing <code>true</code>.
     *
     * @since 2.0b
     */
    private static final String PROPERTY_VALUE_YES = "yes";

    /**
     * Represents the property value of matching all processors flag representing <code>true</code>.
     *
     * @since 2.0b
     */
    private static final String PROPERTY_VALUE_TRUE = "true";

    /**
     * Represents the property value of matching all processors flag representing <code>true</code>.
     *
     * @since 2.0b
     */
    private static final String PROPERTY_VALUE_ON = "on";

    /**
     * Represents the argument option specifying whether blank lines should be ignored or not.
     *
     * @since 2.0b
     */
    private static final ArgumentOption IGNORE_BLANK_LINE_OPTION = new ArgumentOption(
        IGNORE_BLANK_LINE_PROPERTY_KEY, "Whether blank lines should be ignored or not.", false);

    /**
     * <p>
     * Creates a new instance of <code>TextLineCounter</code> class. The alias of this file processor and the file
     * matcher instance used by this file processor are given. The blank lines are not ignored when counting.
     * </p>
     *
     * @param alias the unique alias for the processor.
     * @param matcher the file matcher instance used to determine whether a file should be processed by this file
     *        processor.
     *
     * @throws NullPointerException if <code>alias</code> or <code>matcher</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     */
    public TextLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }


    /**
     * <p>
     * Creates a new instance of <code>TextLineCounter</code> class. The alias of this file processor and the file
     * matcher instance used by this file processor are given. The flag indicating whether the blank lines are
     * ignored or not is given.
     * </p>
     *
     * @param alias the unique alias for the processor.
     * @param matcher the file matcher instance used to determine whether a file should be processed by this file
     *        processor.
     * @param ignoreBlankLine a flag indicating whether blank lines should be ignored when counting.
     *
     * @throws NullPointerException if <code>alias</code> or <code>matcher</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     * @since 2.0b
     */
    public TextLineCounter(String alias, FileMatcher matcher, boolean ignoreBlankLine) {
        this(alias, matcher);

        this.ignoreBlankLine = ignoreBlankLine;
    }

    /**
     * <p>
     * Creates a new instance of <code>TextLineCounter</code> class. The alias and the file matcher are loaded from
     * configuration. The flag indicating whether the blank lines should be ignored or not is loaded as well.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if mandatory property is missing; or the file matcher instance cannot be
     *         created; or the alias is empty string.
     */
    public TextLineCounter(Property property) throws ConfigurationException {
        super(property);

        String value = property.getValue(IGNORE_BLANK_LINE_PROPERTY_KEY);

        // If the value is "on", "yes" or "true", ignoring blank lines should be on.
        ignoreBlankLine = PROPERTY_VALUE_ON.equalsIgnoreCase(value) || PROPERTY_VALUE_TRUE.equalsIgnoreCase(value)
            || PROPERTY_VALUE_YES.equalsIgnoreCase(value);
    }

    /**
     * <p>
     * Gets the list of supported metrics which statistical data are collected against. Each time, a new list should
     * be returned. For this class, the returned list contains only one <code>Metric</code> instance with the name
     * 'Line Count'.
     * </p>
     *
     * @return the list of <code>Metric</code> instances representing supported metrics of this processor.
     */
    public List getSupportedMetrics() {
        List metrics = new ArrayList();

        metrics.add(SUPPORTED_METRIC);

        return metrics;
    }

    /**
     * <p>
     * Processes the given file. The number of lines in the given file is counted.
     * </p>
     *
     * @param file the file where statistical data are collected from.
     *
     * @return the statistical result collected from the given file containing the number of lines in the file.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     */
    public StatsResult process(File file) throws IOException {
        if (file == null) {
            throw new NullPointerException("file cannot be null.");
        }

        long lines = 0;
        BufferedReader input = null;
        String line = null;

        try {
            // Create an buffered text reader using the default encoding
            input = new BufferedReader(new FileReader(file));

            while ((line = input.readLine()) != null) {
                // If the line is non-empty, or the counting does not ignore blank lines, increase the line number.
                if (!ignoreBlankLine || (line.trim().length() != 0)) {
                    ++lines;
                }
            }
        } finally {
            if (input != null) {
                input.close();
            }
        }

        // Create the result
        StatsResult result = new StatsResult();

        // Set the line count
        result.setStat(SUPPORTED_METRIC, lines);

        return result;
    }

    /**
     * <p>
     * Gets a list of custom argument options which can be processed by this processor. Only one optional argument
     * can be processed by this processor, which is 'ignore_blank_line'. When the switch is specified, blank lines
     * are ignored when counting the line number. The value of the switch is ignored.
     * </p>
     *
     * @return a list of <code>ArgumentOption</code> instances representing the switches can be processed by this
     *         processor.
     * @since 2.0b
     */
    public List proposeOptions() {
        List options = new ArrayList();

        options.add(IGNORE_BLANK_LINE_OPTION);

        return options;
    }

    /**
     * <p>
     * Handles the argument received from the command line. When the switch 'ignore_blank_line' is
     * specified, blank lines are ignored when counting line number. The value of the switch is ignored.
     * </p>
     *
     * @param arguments the argument names and values received from the command line.
     * @throws NullPointerException if <code>arguments</code> is <code>null</code>.
     * @throws IllegalArgumentExcpetion if the value of switch 'ignore_blank_line' is not <code>null</code> and
     *         not a string.
     * @since 2.0b
     */
    public void handleArguments(Map arguments) {
        ignoreBlankLine = LineCounterHelper.isSwitchSpecified(arguments, IGNORE_BLANK_LINE_PROPERTY_KEY);
    }
}
