/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;

import com.topcoder.util.config.Property;

import java.io.File;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * Defines an abstract implementation of <code>StatsReporting</code> interface. This abstract class provides
 * functionality to aggregates statistical results for each processor, file and directory. It also provides an
 * overall statistical result. A base directory may be specified when resetting the aggregated statistical results.
 * When it is specified, all directory and file paths are relative to the base directory. Concrete implementations
 * should implement the method generating the statsitics report.
 * </p>
 *
 * <p>
 * This class requires no configuration.
 * </p>
 *
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public abstract class AbstractStatsReporting implements StatsReporting {
    /**
     * Represents the statistical results aggregated for each file. The keys are file paths (relative to base
     * directory if available, or absolute path).
     */
    private StatsCollection fileStats = new StatsCollection();

    /**
     * Represents the statistical results aggregated for each directory. The keys are directory paths (relative to
     * base directory if available, or absolute path). The base directory itself is not included, since it is the
     * same as the overall result.
     */
    private StatsCollection directoryStats = new StatsCollection();

    /**
     * Represents the statistical results aggregated for each file processor. The keys are aliases of file
     * processors.
     */
    private StatsCollection aliasStats = new StatsCollection();

    /** Represents the overall aggregated statistical results. */
    private StatsResult overallStats = new StatsResult();

    /**
     * Represents the base directory used in this class. When it is <code>null</code>, the base directory is not
     * specified.
     */
    private File baseDirectory = null;

    /**
     * <p>
     * Creates a new instance of <code>AbstractStatsReporting</code> class. The base directory is not specified.
     * </p>
     */
    protected AbstractStatsReporting() {
    }

    /**
     * <p>
     * Creates a new instance of <code>AbstractStatsReporting</code> class. The base directory is not specified.
     * Currently, there is no configuration for this class.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if error occurs when loading configuration; or mandatory property is missing;
     *         or invalid property value exists.
     */
    protected AbstractStatsReporting(Property property)
        throws ConfigurationException {
        Validator.validateNotNull(property, "property");
    }

    /**
     * <p>
     * Aggregates the given result processed by the given processor with the given file. The aggregated results for
     * the given file, the given processor and the parent directories of the given file, and the overall result are
     * updated.
     * </p>
     *
     * @param processor the file processor collecting the statistical data.
     * @param file the file where the statistical data are collected from.
     * @param result the statistical result generated by the file processor.
     *
     * @throws NullPointerException if <code>processor</code>, <code>file</code> or <code>result</code> is
     *         <code>null</code>.
     * @throws StatsReportingException if <code>file</code> is not located in the base directory when the base
     *         directory is specified.
     */
    public void aggregate(FileProcessor processor, File file, StatsResult result)
        throws StatsReportingException {
        Validator.validateNotNull(processor, "processor");
        Validator.validateNotNull(file, "file");
        Validator.validateNotNull(result, "result");

        if (baseDirectory != null) {
            // If base directory exists, remove the base portion from the file name
            // Since different platform may compare the file name in a case-sensitive or case-insensitive manner,
            // here it uses File.equals for comparison.
            // The list represents the file name and path name
            List list = new ArrayList();

            while (!file.equals(baseDirectory)) {
                list.add(file.getName());
                file = file.getParentFile();

                if (file == null) {
                    // If the file is null already, the file is not in the base directory.
                    throw new StatsReportingException("The given file is not in the base directory.");
                }
            }

            if (list.size() == 0) {
                // When the given file is the directory, it is not supported.
                throw new StatsReportingException("The given file is the base directory.");
            }

            // Rebuild the relative path.
            file = null;

            for (int nameIndex = list.size() - 1; nameIndex >= 0; --nameIndex) {
                file = new File(file, (String) list.get(nameIndex));
            }
        }

        // Aggregate the alias, file and overall report
        aliasStats.add(processor.getAlias(), result);
        fileStats.add(file.getPath(), result);
        overallStats.aggregate(result);

        // Aggregate the directory report.
        File dir = file.getParentFile();

        // Process until there is no parent directory.
        while (dir != null) {
            directoryStats.add(dir.getPath(), result);

            // Get the parent directory.
            dir = dir.getParentFile();
        }
    }

    /**
     * <p>
     * Clears the current aggregated results. The base directory is specified if <code>baseDirectory</code> is not
     * <code>null</code>. A <code>null</code> value means the base directory is not specified. All processed files
     * should be located in the base directory if specified.
     * </p>
     *
     * @param baseDirectory the base directory where all processed files should be located in if not
     *        <code>null</code>.
     *
     * @throws IllegalArgumentException if <code>baseDirectory</code> is not a valid directory when it is not
     *         <code>null</code>.
     */
    public void reset(File baseDirectory) {
        if (baseDirectory != null) {
            // When the base directory is not null, it must be an existing directory.
            if (!baseDirectory.isDirectory()) {
                throw new IllegalArgumentException("baseDirectory must be an existing directory.");
            }
        }

        // When the checking is done, clear all collections, create a new overall statistics and set the base dir
        fileStats.clear();
        directoryStats.clear();
        aliasStats.clear();
        overallStats = new StatsResult();
        this.baseDirectory = baseDirectory;
    }

    /**
     * <p>
     * Gets the aggregated statistical results for files.
     * </p>
     *
     * @return the aggregated statistical results for files.
     */
    public StatsCollection getFileStats() {
        return fileStats;
    }

    /**
     * <p>
     * Gets the aggregated statistical results for directories.
     * </p>
     *
     * @return the aggregated statistical results for directories.
     */
    public StatsCollection getDirectoryStats() {
        return directoryStats;
    }

    /**
     * <p>
     * Gets the aggregated statistical results for file processors.
     * </p>
     *
     * @return the aggregated statistical results for file processors.
     */
    public StatsCollection getAliasStats() {
        return aliasStats;
    }

    /**
     * <p>
     * Gets the overall aggregated statistical result. A copy is returned.
     * </p>
     *
     * @return the overall aggregated statistical result.
     */
    public StatsResult getOverallStats() {
        return new StatsResult(overallStats);
    }

    /**
     * <p>
     * Gets the base directory. If the base directory is not specified, <code>null</code> is returned.
     * </p>
     *
     * @return the base directory.
     */
    public File getBaseDirectory() {
        return baseDirectory;
    }
}




