/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.util.config.ConfigManager;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * Tests functionality and error cases of <code>Main</code> class. Since error messages and results may be printed to
 * the console, the console is captured when setting up the test cases.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class MainTestCase extends TestCase {
    /** Represents the original console output. */
    private PrintStream console;

    /** Represents the captured console output. */
    private ByteArrayOutputStream output;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /** Represents the list of files to be deleted. */
    private List filesToDelete;

    /**
     * <p>
     * Sets up the test environment. The output to the console is captured in a <code>ByteArrayOutputStream</code>.
     * The configurations are removed. The list of deleting files is created.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        console = System.out;
        output = new ByteArrayOutputStream();

        System.setOut(new PrintStream(output));

        configManager = ConfigManager.getInstance();

        // Remove all namespaces
        Iterator iter = configManager.getAllNamespaces();

        while (iter.hasNext()) {
            configManager.removeNamespace((String) iter.next());
        }

        filesToDelete = new ArrayList();
    }

    /**
     * <p>
     * Cleans up the test environment. The console output is restored. The <code>ByteArrayOutputStream</code> is
     * closed. The configurations are removed. The deleting files are deleted.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        System.setOut(console);

        try {
            output.close();
        } catch (IOException e) {
            // good
        }

        // Remove all namespaces
        Iterator iter = configManager.getAllNamespaces();

        while (iter.hasNext()) {
            configManager.removeNamespace((String) iter.next());
        }

        // Delete files
        iter = filesToDelete.iterator();

        while (iter.hasNext()) {
            ((File) iter.next()).delete();
        }

        filesToDelete = null;
        configManager = null;
        output = null;
        console = null;
    }

    /**
     * <p>
     * Tests <code>main</code> when there is an invalid switch specified. The help message should be printed.
     * </p>
     */
    public void testSwitchInvalid() {
        Main.main(new String[] {"-invalid"});

        assertHelpMessage();
    }

    /**
     * <p>
     * Tests <code>main</code> when there is no file/directory specified to process. The help message should be
     * printed.
     * </p>
     */
    public void testNoFile() {
        Main.main(new String[] {"-r"});

        assertHelpMessage();
    }

    /**
     * <p>
     * Tests <code>main</code> when error occurs during processing. The exception message should be printed. All the
     * causes of the exception are also printed in order to let user know more detail of the error. However, not to
     * confuse end user with too much technical details, the stack trace is not printed.
     * </p>
     */
    public void testError() {
        Main.main(new String[] {"-n", "unknown", "test_files"});

        // Verify error message. Especially the cause of error
        String message = new String(output.toByteArray());

        assertEquals("The error message should be correct.",
            "Error occurred: Loading configuration fails. caused by unknown is unknown", message.trim());
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when no switch is specified. Configuration should be preloaded and use
     * 'com.topcoder.file.statistics.Main' as namespace. No base directory is used. The processing should not be
     * done recursively. The report should be printed to the console. These are verified by the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDefaultAccuracy() throws Exception {
        // Preload the configuration
        configManager.add("Main/Valid.xml");

        // Emulate the command line
        Main.main(new String[] {"test_files/Deep", "test_files/large.txt"});

        // Verify the report
        String report = new String(output.toByteArray());

        TestHelper.assertEquals("The generated report should be correct.", new File("test_files/MainDefault.txt"),
            report);
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when the namespace is specified. Configuration should be preloaded and use
     * the given namespace. No base directory is used. The processing should not be done recursively. The report
     * should be printed to the console. These are verified by the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testNamespaceAccuracy() throws Exception {
        // Preload the configuration
        configManager.add("Main/Namespace.xml");

        // Emulate the command line
        Main.main(new String[] {"-n", "test", "test_files/Deep", "test_files/large.txt"});

        // Verify the report
        String report = new String(output.toByteArray());

        TestHelper.assertEquals("The generated report should be correct.", new File("test_files/MainDefault.txt"),
            report);
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when the configuration file is specified. The specified configuration file
     * should be loaded. Namespace 'com.topcoder.file.statistics.Main' should be used. No base directory is used.
     * The processing should not be done recursively. The report should be printed to the console. These are
     * verified by the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testConfigurationAccuracy() throws Exception {
        // Emulate the command line
        Main.main(new String[] {"-c", "Main/Valid.xml", "test_files/Deep", "test_files/large.txt"});

        // Verify the report
        String report = new String(output.toByteArray());

        TestHelper.assertEquals("The generated report should be correct.", new File("test_files/MainDefault.txt"),
            report);
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when the configuration file and namespace are specified. The specified
     * configuration file should be loaded to the specified namespace, which should be used. No base directory is
     * used. The processing should not be done recursively. The report should be printed to the console. These are
     * verified by the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testConfigurationNamespaceAccuracy() throws Exception {
        // Emulate the command line
        Main.main(new String[] {"-c", "Main/Load.xml", "-n", "test", "test_files/Deep", "test_files/large.txt"});

        // Verify the report
        String report = new String(output.toByteArray());

        TestHelper.assertEquals("The generated report should be correct.", new File("test_files/MainDefault.txt"),
            report);
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when a custom switch is specified. The custom switch should be passed to
     * proper argument handler. In this case, the text line counter should ignore the blank lines, which makes the
     * line number 530 instead of 579.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSpecifyCustomArgumentAccuracy() throws Exception {
        // Preload the configuration
        configManager.add("Main/Valid.xml");

        // Emulate the command line, which set the ignore blank line flag to true
        Main.main(new String[] {"-ignore_blank_line", "", "test_files/large.txt"});

        // Verify the report
        String report = new String(output.toByteArray());

        TestHelper.assertEquals("The line count should be 530, not 579.", new File("test_files/CustomSwitch.txt"),
            report);
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when the recursive switch is specified. Configuration should be preloaded
     * and use the given namespace. No base directory is used. The processing should be done recursively. The report
     * should be printed to the console. These are verified by the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRecursiveAccuracy() throws Exception {
        // Preload the configuration
        configManager.add("Main/Valid.xml");

        // Emulate the command line
        Main.main(new String[] {"-r", "test_files/Deep", "test_files/large.txt"});

        // Verify the report
        String report = new String(output.toByteArray());

        TestHelper.assertEquals("The generated report should be correct.", new File("test_files/MainRecursive.txt"),
            report);
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when the base directory is specified. Configuration should be preloaded
     * and use the given namespace. The specified base directory is used. The processing should be done recursively.
     * The report should be printed to the console. These are verified by the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testBaseAccuracy() throws Exception {
        // Preload the configuration
        configManager.add("Main/Valid.xml");

        // Emulate the command line
        Main.main(new String[] {"-b", "test_files", "Deep", "large.txt"});

        // Verify the report
        String report = new String(output.toByteArray());

        TestHelper.assertEquals("The generated report should be correct.", new File("test_files/MainBase.txt"), report);
    }

    /**
     * <p>
     * Tests accuracy of <code>main</code> when the output file is specified. Configuration should be preloaded and
     * use the given namespace. No base directory is used. The processing should be done recursively. The report
     * should be written to the specified file. These are verified by the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testOutputFileAccuracy() throws Exception {
        // Preload the configuration
        configManager.add("Main/Valid.xml");

        File file = new File("test_files/report.tmp");

        filesToDelete.add(file);

        // Emulate the command line
        Main.main(new String[] {"-o", "test_files/report.tmp", "test_files/Deep", "test_files/large.txt"});

        // Read the generated file
        InputStream input = null;
        byte[] content;

        try {
            input = new FileInputStream(file);
            content = new byte[(int) file.length()];

            input.read(content);
        } finally {
            if (input != null) {
                input.close();
            }
        }

        // Get the report and verify
        String report = new String(content);

        TestHelper.assertEquals("The generated report should be correct.", new File("test_files/MainDefault.txt"),
            report);
    }

    /**
     * <p>
     * Asserts that the help message is printed to the console.
     * </p>
     */
    private void assertHelpMessage() {
        String message = new String(output.toByteArray());

        assertTrue("The help message should be printed.", message.indexOf("Usage:") >= 0);
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(MainTestCase.class);
    }
}




