/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.accuracytests.matcher;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import junit.framework.TestCase;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.accuracytests.TestHelper;
import com.topcoder.file.statistics.matcher.RegexMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;


/**
 * <p>
 * Accuracy tests for RegexMatcher.
 * </p>
 *
 * @author cucu
 * @version 1.0
 */
public class RegexMatcherAccuracyTests extends TestCase {
    /**
     * Test the Match method against a single pattern.
     */
    public void testMatchWithStringConstructor() throws Exception  {
        FileMatcher matcher = new RegexMatcher(Pattern.compile("^file[0-9][0-9]"));

        assertTrue("file99 should match with the regex", matcher.match(new File("file99")));
        assertTrue("file12.txt should match with the regex", matcher.match(new File("file12.txt")));
        assertTrue("/file00.cpp.bak should match with the regex", matcher.match(new File("/file00.cpp.bak ")));
        assertTrue("c:/doc/file34.txt should match with the regex", matcher.match(new File("c:/doc/file34.txt")));

        assertFalse("myfile00.cpp should not match with the regex", matcher.match(new File("myfile00.cpp")));
        assertFalse("FiLe00.cpp should not match with the regex", matcher.match(new File("FiLe00.cpp")));
        assertFalse("FiLe00.cpp\\efile23.abc should not match with the regex", matcher.match(new File("FiLe00.cpp\\efile23.abc")));
    }


    /**
     * Test that the list constructor makes a copy of the list
     */
    public void testListConstructor() throws Exception {

        List patterns  = new ArrayList();
        patterns.add(Pattern.compile("x"));

        FileMatcher matcher = new RegexMatcher(patterns);

        // this shouldn't affect the matcher
        patterns.add(Pattern.compile("a"));

        assertFalse("constructor is not making a copy", matcher.match(new File("abc")));
    }

    /**
     * Test the Match method against many patterns, using the constructor that takes a list of Regex.
     */
    public void testMatchWithListConstructor() throws Exception  {
        List patterns = new ArrayList();
        patterns.add(Pattern.compile("\\.mp[\\d]$"));
        patterns.add(Pattern.compile("TopCoder"));
        patterns.add(Pattern.compile("hello"));

        FileMatcher matcher = new RegexMatcher(patterns);

        checkPatterns(matcher);
    }

    /**
     *  Test the Match method against many extensions, using the constructor that reads from the configuration file.
     *
     * @throws Exception to JUnit
     */
    public void testMatchWithConfigManagerConstructor() throws Exception  {
        TestHelper.loadConfig(TestHelper.MATCHERS_CONFIG);

        try {
            Property prop = ConfigManager.getInstance().getPropertyObject(TestHelper.MATCHER_NAMESPACE, "regex");
            FileMatcher matcher = new RegexMatcher(prop);
            checkPatterns(matcher);
        } finally {
            TestHelper.unloadNamespace(TestHelper.MATCHER_NAMESPACE);
        }
    }

    /**
     * Check that the matcher passed is matching well with the extensions: cpp, cs, java and pas.
     *
     * @param matcher the matcher to check
     */
    private void checkPatterns(FileMatcher matcher) throws Exception {
        assertTrue("music.mp3 should match with the patterns", matcher.match(new File("music.mp3")));
        assertTrue("e:\\music.mp4 should match with the patterns", matcher.match(new File("e:\\music.mp4")));
        assertTrue("TopCoder.txt should match with the patterns", matcher.match(new File("TopCoder.txt")));
        assertTrue("say_hello_world.txt should match with the patterns", matcher.match(new File("say_hello_world.txt")));

        assertFalse("music.mp3.bak should NOT match with the patterns", matcher.match(new File("music.mp3.bak")));
        assertFalse("/music.mpx should NOT match with the patterns", matcher.match(new File("/music.mpx")));
        assertFalse("topcoder.txt should NOT match with the patterns", matcher.match(new File("topcoder.txt")));

    }

}
