/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.io.IOException;


/**
 * <p>
 * Tests functionality and error cases of <code>FileCategoryMatcher</code> class. Since it is an abstract class, a
 * mock concrete implementation <code>MockFileCategoryMatcher</code> is used.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class FileCategoryMatcherTestCase extends TestCase {
    /** Represents the <code>FileCategoryMatcher</code> instance used in tests. */
    private MockFileCategoryMatcher matcher;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     */
    protected void setUp() {
        matcher = new MockFileCategoryMatcher();
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed.
     * </p>
     */
    protected void tearDown() {
        matcher = null;
    }

    /**
     * <p>
     * Tests <code>isBinary</code> when the given file is <code>null</code>. Expect NullPointerException
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIsBinaryNull() throws Exception {
        try {
            FileCategoryMatcher.isBinary(null);
            fail("The file is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>isBinary</code> when I/O error occurs. Expect IOException.
     * </p>
     */
    public void testIsBinaryIOError() {
        try {
            FileCategoryMatcher.isBinary(new File("test_files/notexist"));
            fail("I/O error occurs, should throw IOException.");
        } catch (IOException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>match</code> when the given file is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchNull() throws Exception {
        try {
            matcher.match(null);
            fail("The file is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>match</code> when I/O error occurs. Expect IOException.
     * </p>
     */
    public void testMatchIOError() {
        try {
            matcher.match(new File("test_files/notexist"));
            fail("I/O error occurs, should throw IOException.");
        } catch (IOException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>isBinary</code> when the given file is a text file smaller than 1024 bytes. The file
     * should be recognized as text file correctly.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIsBinarySmallTextAccuracy() throws Exception {
        assertFalse("The small text file (<1024 bytes) should be recognized.",
            FileCategoryMatcher.isBinary(new File("test_files/small.txt")));
    }

    /**
     * <p>
     * Tests accuracy of <code>isBinary</code> when the given file is a text file larger than 1024 bytes. The file
     * should be recognized as text file correctly.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIsBinaryLargeTextAccuracy() throws Exception {
        assertFalse("The large text file (>1024 bytes) should be recognized.",
            FileCategoryMatcher.isBinary(new File("test_files/large.txt")));
    }

    /**
     * <p>
     * Tests accuracy of <code>isBinary</code> when the given file is a binary file smaller than 1024 bytes. The file
     * should be recognized as binary file correctly.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIsBinarySmallBinaryAccuracy() throws Exception {
        assertTrue("The small binary file (<1024 bytes) should be recognized.",
            FileCategoryMatcher.isBinary(new File("test_files/small.bin")));
    }

    /**
     * <p>
     * Tests accuracy of <code>isBinary</code> when the given file is a binary file larger than 1024 bytes. The file
     * should be recognized as binary file correctly.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIsBinaryLargeBinaryAccuracy() throws Exception {
        assertTrue("The large binary file (>1024 bytes) should be recognized.",
            FileCategoryMatcher.isBinary(new File("test_files/large.bin")));
    }

    /**
     * <p>
     * Tests accuracy of <code>isBinary</code> when the given file is an empty file. The file should be recognized as
     * text file correctly.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIsBinaryEmptyAccuracy() throws Exception {
        assertFalse("The empty file should be recognized as text file.",
            FileCategoryMatcher.isBinary(new File("test_files/empty")));
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given file is a text file. The protected method
     * <code>match</code> should receive <code>false</code> as argument. The returned value should be the same as
     * the returned value of protected <code>match</code> method.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchTextAccuracy() throws Exception {
        // MockFileCategoryMatcher always returns false
        assertFalse("The returned value should be correct.", matcher.match(new File("test_files/small.txt")));

        // The passed argument should be correct.
        assertFalse("The protected method's argument should be correct.", matcher.getMatchArgument());
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given file is a binary file. The protected method
     * <code>match</code> should receive <code>true</code> as argument. The returned value should be the same as the
     * returned value of protected <code>match</code> method.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchBinaryAccuracy() throws Exception {
        // MockFileCategoryMatcher always returns false
        assertFalse("The returned value should be correct.", matcher.match(new File("test_files/small.bin")));

        // The passed argument should be correct.
        assertTrue("The protected method's argument should be correct.", matcher.getMatchArgument());
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(FileCategoryMatcherTestCase.class);
    }
}




