/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatchingException;

import com.topcoder.util.config.ConfigManager;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.io.IOException;


/**
 * <p>
 * Tests functionality and error cases of <code>FileTypeMatcher</code> class. Since it is an abstract class, a mock
 * concrete implementation <code>MockFileTypeMatcher</code> is used.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class FileTypeMatcherTestCase extends TestCase {
    /** Represents the namespace in configuration used by magic number component. */
    private static final String MAGICNUMBER_NAMESPACE = "com.topcoder.file.type.MagicNumbers.xml";

    /** Represents the <code>FileTypeMatcher</code> instance used in tests. */
    private MockFileTypeMatcher matcher;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test instance is created. The configuration for magic number is loaded.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        configManager = ConfigManager.getInstance();

        if (configManager.existsNamespace(MAGICNUMBER_NAMESPACE)) {
            configManager.removeNamespace(MAGICNUMBER_NAMESPACE);
        }

        configManager.add("MagicNumbers.xml");

        matcher = new MockFileTypeMatcher();
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. The configuration for magic number is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(MAGICNUMBER_NAMESPACE)) {
            configManager.removeNamespace(MAGICNUMBER_NAMESPACE);
        }

        configManager = null;
        matcher = null;
    }

    /**
     * <p>
     * Tests <code>match</code> when the given file is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchNull() throws Exception {
        try {
            matcher.match((File) null);
            fail("The file is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>match</code> when I/O error occurs when matching the file. Expect IOException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchIOError() throws Exception {
        try {
            matcher.match(new File("notexist"));
            fail("I/O error occurs when matching the file, should throw IOException.");
        } catch (IOException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>match</code> when magic number error occurs. Expect FileMatchingException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchMagicNumberError() throws Exception {
        configManager.removeNamespace(MAGICNUMBER_NAMESPACE);
        configManager.add("FileTypeMatcher/WrongMagicNumbers.xml");

        try {
            matcher.match(new File("test_files/small.rar"));
            fail("Magic number error occurs, should throw FileMatchingException.");
        } catch (FileMatchingException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>match</code> when error occurs in the protected <code>match</code> method. Expect
     * FileMatchingException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchProtectedMethodError() throws Exception {
        try {
            matcher.match(new File("test_files/java.class"));
            fail("Error occurs in the protected match method, should throw FileMatchingException.");
        } catch (FileMatchingException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the file type can be recognized. The protected method
     * <code>match</code> should receive the file name as argument. The returned value should be the same as the
     * returned value of protected <code>match</code> method.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchTypeExistAccuracy() throws Exception {
        // MockFileNameMatcher always returns false
        assertFalse("The returned value should be correct.", matcher.match(new File("test_files/small.rar")));

        // The passed argument should be correct.
        assertEquals("The protected method's argument should be correct.", "RAR archive",
            matcher.getMatchArgument().getName());
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the file type cannot be recognized. <code>false</code> should be
     * returned immediately without passing the argument to the protected <code>match</code> method.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMatchTypeUnknownAccuracy() throws Exception {
        // FileNameMatcher should return false immediately
        assertFalse("The returned value should be correct.", matcher.match(new File("test_files/unknown")));

        // No passed argument should be received
        assertNull("No argument should be received when the file type cannot be recognized",
            matcher.getMatchArgument());
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(FileTypeMatcherTestCase.class);
    }
}




