/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.SignatureChecker;
import com.topcoder.file.type.TextSignatureChecker;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.lang.reflect.Field;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * <p>
 * Tests functionality and error cases of <code>TypeNameMatcher</code> class. The <code>match(File)</code> method is
 * defined in its super class <code>FileTypeMatcher</code>. The accuracy and error cases of this method is already
 * tested in the test case of that class. Therefore, here only the protected <code>match</code> method is tested.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class TypeNameMatcherTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the property name used in tests. */
    private static final String PROPERTY = "test_property";

    /** Represents the <code>TypeNameMatcher</code> instance used in tests. */
    private TypeNameMatcher matcher;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test instance is created. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        configManager = ConfigManager.getInstance();

        // The matcher instance contains three type names
        List list = new ArrayList();

        list.add("RAR");
        list.add("Java class");
        list.add("JPEG");
        matcher = new TypeNameMatcher(list);

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        matcher = null;
        configManager = null;
    }

    /**
     * <p>
     * Tests constructor <code>TypeNameMatcher(String)</code> when the given type name is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testTypeNameMatcherStringNull() throws Exception {
        try {
            new TypeNameMatcher((String) null);
            fail("The type name is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>TypeNameMatcher(List)</code> when the given type name list is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     */
    public void testTypeNameMatcherListNull() {
        try {
            new TypeNameMatcher((List) null);
            fail("The type name list is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>TypeNameMatcher(List)</code> when the given type name list is empty. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testTypeNameMatcherListEmpty() {
        try {
            new TypeNameMatcher(new ArrayList());
            fail("The type name list is empty, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>TypeNameMatcher(List)</code> when the given type name list contains empty string.
     * Expect IllegalArgumentException.
     * </p>
     */
    public void testTypeNameMatcherListEmptyString() {
        List list = new ArrayList();

        list.add("  ");

        try {
            new TypeNameMatcher(list);
            fail("The type name list contains empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>TypeNameMatcher(List)</code> when the given type name list contains non-string
     * instance. Expect IllegalArgumentException.
     * </p>
     */
    public void testTypeNameMatcherListNotString() {
        List list = new ArrayList();

        list.add(new Object());

        try {
            new TypeNameMatcher(list);
            fail("The type name list contains non-string instance, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>TypeNameMatcher(Property)</code> when the given property is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testTypeNameMatcherPropertyNull() throws Exception {
        try {
            new TypeNameMatcher((Property) null);
            fail("The property is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>TypeNameMatcher(Property)</code> when the type name list is missing in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testTypeNameMatcherPropertyExtensionMissing()
        throws Exception {
        configManager.add("TypeNameMatcher/Missing.xml");

        try {
            new TypeNameMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The type name list is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(Property)</code> when the type name is empty string in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPropertyPatternEmpty()
        throws Exception {
        configManager.add("TypeNameMatcher/EmptyString.xml");

        try {
            new RegexMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The type name is empty string in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>TypeNameMatcher(String)</code>. The internal type name set should contain
     * only one type name, which is given as the argument.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testTypeNameMatcherStringAccuracy() throws Exception {
        matcher = new TypeNameMatcher("RAR");

        // Get the internal set
        Field field = TypeNameMatcher.class.getDeclaredField("types");
        Set list;

        try {
            field.setAccessible(true);
            list = (Set) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Verify
        assertEquals("One type name should exist.", 1, list.size());
        assertTrue("The type name should be correct.", list.contains("RAR"));
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>TypeNameMatcher(List)</code>. The internal type name set should contain
     * all type names which are given as the argument. A copy of the given list should be made.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testTypeNameMatcherListAccuracy() throws Exception {
        List list = new ArrayList();

        list.add("RAR");
        list.add("Java class");
        list.add("JPEG");

        matcher = new TypeNameMatcher(list);

        // Get the internal set
        Field field = TypeNameMatcher.class.getDeclaredField("types");
        Set actual;

        try {
            field.setAccessible(true);
            actual = (Set) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Build the expected set
        Set expected = new HashSet(list);

        // Verify copy
        assertNotSame("A copy of the list should be made.", list, actual);

        // Verify the set
        assertEquals("The type name set should be correct.", expected, actual);
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>TypeNameMatcher(Property)</code>. The internal type name set should
     * contain all type names which are specified in the configuration.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testTypeNameMatcherPropertyAccuracy() throws Exception {
        configManager.add("TypeNameMatcher/Valid.xml");

        matcher = new TypeNameMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        // Get the internal set
        Field field = TypeNameMatcher.class.getDeclaredField("types");
        Set actual;

        try {
            field.setAccessible(true);
            actual = (Set) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Build the expected list set.
        Set expected = new HashSet();

        expected.add("RAR");
        expected.add("Java class");
        expected.add("JPEG");

        // Verify the set
        assertEquals("The type name set should be correct.", expected, actual);
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given type name exists in the type name list. The comparison
     * should be done case-sensitively.
     * </p>
     */
    public void testMatchStringExistAccuracy() {
        FileType type = new FileType("RAR", new String[0], new String[0], "application/x-rar",
                new SignatureChecker[] {new TextSignatureChecker()});

        assertTrue("The type name exists in the type name list.", matcher.match(type));
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given type name does not exist in the type name list. The
     * comparison should be done case-sensitively.
     * </p>
     */
    public void testMatchStringNotExistAccuracy() {
        FileType type = new FileType("RaR", new String[0], new String[0], "application/x-RaR",
                new SignatureChecker[] {new TextSignatureChecker()});

        assertFalse("The type name does not exist in the type name list.", matcher.match(type));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(TypeNameMatcherTestCase.class);
    }
}

