/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.processor;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.MimeMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;

import com.topcoder.util.config.ConfigManager;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * <p>
 * Tests functionality and error cases of <code>FileProcessorBase</code> class. This class is an abstract class, so
 * its concrete implementation <code>TextLineCounter</code> is used for testing purposes, since the sub-class
 * delegates the super class' constructors, and does not override any methods in the super class. The accuracy
 * tests of property getters are combined into the accuracy tests of constructors, since this class is immutable.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class FileProcessorBaseTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the property name used in tests. */
    private static final String PROPERTY = "test_property";

    /** Represents the file matcher instance used in tests. */
    private static final FileMatcher FILE_MATCHER = new AnyFileMatcher();

    /** Represents a valid alias used in tests. */
    private static final String ALIAS = "alias";

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        configManager = ConfigManager.getInstance();

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        configManager = null;
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(String, FileMatcher)</code> when the alias is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     */
    public void testFileProcessorBaseStringFileMatcherAliasNull() {
        try {
            new TextLineCounter(null, FILE_MATCHER);
            fail("The alias is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(String, FileMatcher)</code> when the alias is empty string. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testFileProcessorBaseStringFileMatcherAliasEmpty() {
        try {
            new TextLineCounter("  ", FILE_MATCHER);
            fail("The alias is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(String, FileMatcher)</code> when the file matcher is
     * <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testFileProcessorBaseStringFileMatcherFileMatcherNull() {
        try {
            new TextLineCounter(ALIAS, null);
            fail("The file matcher is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>FileProcessorBase(String, FileMatcher)</code>. The alias and the file
     * matcher of the new created instance should be set as the alias and file matcher given in the arguments.
     * </p>
     */
    public void testFileProcessorBaseStringFileMatcherAccuracy() {
        FileProcessorBase processor = new TextLineCounter(ALIAS, FILE_MATCHER);

        // Verify properties
        assertEquals("The alias should be correct.", ALIAS, processor.getAlias());
        assertSame("The file matcher should be correct.", FILE_MATCHER, processor.getFileMatcher());
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(Property)</code> when the given property is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileProcessorBasePropertyNull() throws Exception {
        try {
            new TextLineCounter(null);
            fail("The property is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(Property)</code> when the alias is missing in configuration. Expect
     * ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileProcessorBasePropertyAliasMissing()
        throws Exception {
        configManager.add("FileProcessorBase/AliasMissing.xml");

        try {
            new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The alias is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(Property)</code> when the alias is empty string in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileProcessorBasePropertyAliasEmpty()
        throws Exception {
        configManager.add("FileProcessorBase/AliasEmpty.xml");

        try {
            new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The alias is empty string in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(Property)</code> when the file matcher class name is missing in
     * configuration. Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileProcessorBasePropertyMatcherClassMissing()
        throws Exception {
        configManager.add("FileProcessorBase/MatcherClassMissing.xml");

        try {
            new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The file matcher class name is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileProcessorBase(Property)</code> when the file matcher instance cannot be created.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileProcessorBasePropertyMatcherError()
        throws Exception {
        configManager.add("FileProcessorBase/MatcherError.xml");

        try {
            new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The file matcher instance cannot be created, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>FileProcessorBase(Property)</code> when the matcher is specified in the
     * configuration. The file matcher type should be correct as specified in the configuration. The alias should
     * also be correct as specified in the configuration.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileProcessorBasePropertyMatcherSpecifiedAccuracy()
        throws Exception {
        configManager.add("FileProcessorBase/Valid.xml");

        FileProcessorBase processor = new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        // Verify the properties
        assertEquals("The alias should be the same as specified in the configuration.", ALIAS, processor.getAlias());
        assertTrue("The file matcher should be MimeMatcher.", processor.getFileMatcher() instanceof MimeMatcher);
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>FileProcessorBase(Property)</code> when the matcher is missing in the
     * configuration. The instance should use <code>AnyFileMatcher</code>. The alias should also be correct as
     * specified in the configuration.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileProcessorBasePropertyMatcherMissingAccuracy()
        throws Exception {
        configManager.add("FileProcessorBase/MatcherMissing.xml");

        FileProcessorBase processor = new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        // Verify the properties
        assertEquals("The alias should be the same as specified in the configuration.", ALIAS, processor.getAlias());
        assertTrue("The file matcher should be AnyFileMatcher.", processor.getFileMatcher() instanceof AnyFileMatcher);
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(FileProcessorBaseTestCase.class);
    }
}




