/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.util.config.ConfigManager;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Tests functionality and error cases of <code>TextLineCounter</code> class. Since the constructors of this class
 * soly delegate the super class' constructors, the tests of constructors are not needed.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class TextLineCounterTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the property name used in tests. */
    private static final String PROPERTY = "test_property";

    /** Represents the <code>TextLineCounter</code> instance used in tests. */
    private TextLineCounter processor;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test instance is created using <code>AnyFileMatcher</code>.
     * Configurations are removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        processor = new TextLineCounter("alias", new AnyFileMatcher());

        configManager = ConfigManager.getInstance();

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. Configurations are removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        processor = null;

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        configManager = null;
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>TextLineCounter(String, FileMatcher, boolean)</code>. Since it delegates
     * the super-class's constructor, only the ignore blank lines flag is tested. The ignore blank lines flag should
     * be the same as give.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     * @since 2.0b
     */
    public void testTextLineCounterStringFileMatcherBooleanAccuracy() throws Exception {
        Field field = TextLineCounter.class.getDeclaredField("ignoreBlankLine");

        // Handle the argument
        processor = new TextLineCounter("alias", new AnyFileMatcher(), true);

        try {
            field.setAccessible(true);

            assertEquals("The blank lines should be the same as set.", Boolean.TRUE, field.get(processor));
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>TextLineCounter(Property)</code>. Since it delegates the super-class's
     * constructor, only the ignore blank lines flag is tested. The ignore blank lines flag should <code>true</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     * @since 2.0b
     */
    public void testTextLineCounterPropertyAccuracy() throws Exception {
        configManager.add("TextLineCounter/Valid.xml");

        Field field = TextLineCounter.class.getDeclaredField("ignoreBlankLine");

        processor = new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        try {
            field.setAccessible(true);

            assertEquals("The blank lines should be ignored.", Boolean.TRUE, field.get(processor));
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>TextLineCounter(Property)</code>. Since it delegates the super-class's
     * constructor, only the ignore blank lines flag is tested. The ignore blank lines flag should <code>true</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     * @since 2.0b
     */
    public void testTextLineCounterPropertyMissingAccuracy() throws Exception {
        configManager.add("TextLineCounter/Missing.xml");

        Field field = TextLineCounter.class.getDeclaredField("ignoreBlankLine");

        processor = new TextLineCounter(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        try {
            field.setAccessible(true);

            assertEquals("The blank lines should not be ignored.", Boolean.FALSE, field.get(processor));
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests <code>process</code> when the given file is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessNull() throws Exception {
        try {
            processor.process(null);
            fail("The file is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>process</code> when I/O error occurs when processing the file. Expect IOException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessIOError() throws Exception {
        try {
            processor.process(new File("notexist"));
            fail("I/O error occurs when processing the file, should throw IOException.");
        } catch (IOException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>getSupportedMetrics</code>. A list containing only one metric with the name 'Line
     * Count' and display name 'line' should be returned.
     * </p>
     */
    public void testGetSupportedMetricsAccuracy() {
        List list = processor.getSupportedMetrics();

        // Verify the size of list
        assertEquals("The metric list should contain only 1 metric.", 1, list.size());

        // Verify the content of metric
        Metric metric = (Metric) list.get(0);

        assertEquals("The name of the metric should be correct.", "Line Count", metric.getName());
        assertEquals("The display name of the metric should be correct.", "line", metric.getDisplayName());
        assertEquals("The description of the metric should be correct.",
            "Count the number of lines for a text file.", metric.getDescription());
    }

    /**
     * <p>
     * Tests accuracy of <code>getSupportedMetrics</code>. A copy of the list should be returned.
     * </p>
     */
    public void testGetSupportedMetricsCopyAccuracy() {
        List list = processor.getSupportedMetrics();

        assertNotSame("A copy of the list should be returned.", list, processor.getSupportedMetrics());
    }

    /**
     * <p>
     * Tests accuracy of <code>process</code>. The returned statistical result should contain only one metric. The
     * correct number of lines should be returned. The blank lines should not be ignored.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessAccuracy() throws Exception {
        Metric metric = (Metric) processor.getSupportedMetrics().get(0);
        StatsResult result = processor.process(new File("test_files/strange.c"));

        // Verify metrics
        assertEquals("The statistical result should contain only one metric.", processor.getSupportedMetrics(),
            result.getMetrics());

        // Verify result
        assertEquals("The number of lines should be correct.", 14, result.getStat(metric));
    }

    /**
     * <p>
     * Tests <code>handleArguments</code> when the given argument map is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     * @since 2.0b
     */
    public void testHandleArgumentsNull() throws Exception {
        try {
            processor.handleArguments(null);
            fail("The arguments is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>handleArguments</code> when the 'ignore_blank_line' value is not string. Expect
     * IllegalArgumentException.
     * </p>
     *
     * @since 2.0b
     */
    public void testHandleArgumentsNotString() {
        Map map = new HashMap();

        map.put("ignore_blank_line", new Object());

        try {
            processor.handleArguments(map);
            fail("The 'stylesheet' value is not string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>proposeOptions</code>. The returned list should contain only one argument option of
     * 'ignore_blank_line'. It should be optional, and should have no validator.
     * </p>
     *
     * @since 2.0b
     */
    public void testProposeOptionsAccuracy() {
        List list = processor.proposeOptions();

        assertEquals("The size of the list should be 1.", 1, list.size());

        ArgumentOption option = (ArgumentOption) list.get(0);

        // Verify name, required flag and validator
        assertEquals("The name should be 'ignore_blank_line'.", "ignore_blank_line", option.getSwitchName());
        assertFalse("The switch should be optional.", option.isRequired());
        assertNull("The switch should have no validator.", option.getValidator());
    }

    /**
     * <p>
     * Tests accuracy of <code>handleArguments</code> when the 'ignore_blank_line' switch is not specified. The blank
     * lines should not be ignored.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     * @since 2.0b
     */
    public void testHandleArgumentsNotSpecifiedAccuracy() throws Exception {
        Field field = TextLineCounter.class.getDeclaredField("ignoreBlankLine");

        // Handle the argument
        processor.handleArguments(new HashMap());

        try {
            field.setAccessible(true);

            assertEquals("The blank lines should not be ignored.", Boolean.FALSE, field.get(processor));
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>handleArguments</code> when the 'ignore_blank_line' switch is specified. The blank lines
     * should be ignored. The value should be ignored.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     * @since 2.0b
     */
    public void testHandleArgumentsSpecifiedAccuracy() throws Exception {
        Field field = TextLineCounter.class.getDeclaredField("ignoreBlankLine");
        Map map = new HashMap();

        map.put("ignore_blank_line", "any");

        // Handle the argument
        processor.handleArguments(map);

        try {
            field.setAccessible(true);

            assertEquals("The blank lines should be ignored.", Boolean.TRUE, field.get(processor));
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>process</code>. The returned statistical result should contain only one metric. The
     * correct number of lines should be returned. The blank lines should be ignored.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     * @since 2.0b
     */
    public void testProcessIgnoreBlankLinesAccuracy() throws Exception {
        processor = new TextLineCounter("alias", new AnyFileMatcher(), true);

        Metric metric = (Metric) processor.getSupportedMetrics().get(0);
        StatsResult result = processor.process(new File("test_files/strange.c"));

        // Verify metrics
        assertEquals("The statistical result should contain only one metric.", processor.getSupportedMetrics(),
            result.getMetrics());

        // Verify result
        assertEquals("The number of lines should be correct.", 12, result.getStat(metric));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(TextLineCounterTestCase.class);
    }
}




